# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
import re
import uuid
from collections import OrderedDict
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

import google.protobuf
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.policysimulator_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

import google.api_core.operation as operation  # type: ignore
import google.api_core.operation_async as operation_async  # type: ignore
import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore

from google.cloud.policysimulator_v1.services.org_policy_violations_preview_service import (
    pagers,
)
from google.cloud.policysimulator_v1.types import orgpolicy as gcp_orgpolicy

from .client import OrgPolicyViolationsPreviewServiceClient
from .transports.base import (
    DEFAULT_CLIENT_INFO,
    OrgPolicyViolationsPreviewServiceTransport,
)
from .transports.grpc_asyncio import (
    OrgPolicyViolationsPreviewServiceGrpcAsyncIOTransport,
)

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class OrgPolicyViolationsPreviewServiceAsyncClient:
    """Violations Preview API service for OrgPolicy.

    An
    [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
    is a preview of the violations that will exist as soon as a proposed
    OrgPolicy change is submitted. To create an
    [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview],
    the API user specifies the changes they wish to make and requests
    the generation of a preview via [GenerateViolationsPreview][]. the
    OrgPolicy Simulator service then scans the API user's currently
    existing resources to determine these resources violate the newly
    set OrgPolicy.
    """

    _client: OrgPolicyViolationsPreviewServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = OrgPolicyViolationsPreviewServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        OrgPolicyViolationsPreviewServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    _DEFAULT_ENDPOINT_TEMPLATE = (
        OrgPolicyViolationsPreviewServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    )
    _DEFAULT_UNIVERSE = OrgPolicyViolationsPreviewServiceClient._DEFAULT_UNIVERSE

    custom_constraint_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.custom_constraint_path
    )
    parse_custom_constraint_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_custom_constraint_path
    )
    org_policy_violation_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.org_policy_violation_path
    )
    parse_org_policy_violation_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_org_policy_violation_path
    )
    org_policy_violations_preview_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.org_policy_violations_preview_path
    )
    parse_org_policy_violations_preview_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_org_policy_violations_preview_path
    )
    policy_path = staticmethod(OrgPolicyViolationsPreviewServiceClient.policy_path)
    parse_policy_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_policy_path
    )
    common_billing_account_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OrgPolicyViolationsPreviewServiceAsyncClient: The constructed client.
        """
        sa_info_func = (
            OrgPolicyViolationsPreviewServiceClient.from_service_account_info.__func__  # type: ignore
        )
        return sa_info_func(
            OrgPolicyViolationsPreviewServiceAsyncClient, info, *args, **kwargs
        )

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OrgPolicyViolationsPreviewServiceAsyncClient: The constructed client.
        """
        sa_file_func = (
            OrgPolicyViolationsPreviewServiceClient.from_service_account_file.__func__  # type: ignore
        )
        return sa_file_func(
            OrgPolicyViolationsPreviewServiceAsyncClient, filename, *args, **kwargs
        )

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return (
            OrgPolicyViolationsPreviewServiceClient.get_mtls_endpoint_and_cert_source(
                client_options
            )
        )  # type: ignore

    @property
    def transport(self) -> OrgPolicyViolationsPreviewServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            OrgPolicyViolationsPreviewServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self) -> str:
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = OrgPolicyViolationsPreviewServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                OrgPolicyViolationsPreviewServiceTransport,
                Callable[..., OrgPolicyViolationsPreviewServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the org policy violations preview service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,OrgPolicyViolationsPreviewServiceTransport,Callable[..., OrgPolicyViolationsPreviewServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the OrgPolicyViolationsPreviewServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = OrgPolicyViolationsPreviewServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.cloud.policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient`.",
                extra={
                    "serviceName": "google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService",
                    "credentialsType": None,
                },
            )

    async def list_org_policy_violations_previews(
        self,
        request: Optional[
            Union[gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListOrgPolicyViolationsPreviewsAsyncPager:
        r"""ListOrgPolicyViolationsPreviews lists each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        in an organization. Each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        is available for at least 7 days.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_list_org_policy_violations_previews():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.ListOrgPolicyViolationsPreviewsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_org_policy_violations_previews(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.ListOrgPolicyViolationsPreviewsRequest, dict]]):
                The request object. ListOrgPolicyViolationsPreviewsRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].
            parent (:class:`str`):
                Required. The parent the violations are scoped to.
                Format:
                ``organizations/{organization}/locations/{location}``

                Example:
                ``organizations/my-example-org/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.pagers.ListOrgPolicyViolationsPreviewsAsyncPager:
                ListOrgPolicyViolationsPreviewsResponse is the response message for
                   [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest
        ):
            request = gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_org_policy_violations_previews
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListOrgPolicyViolationsPreviewsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_org_policy_violations_preview(
        self,
        request: Optional[
            Union[gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> gcp_orgpolicy.OrgPolicyViolationsPreview:
        r"""GetOrgPolicyViolationsPreview gets the specified
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].
        Each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        is available for at least 7 days.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_get_org_policy_violations_preview():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.GetOrgPolicyViolationsPreviewRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_org_policy_violations_preview(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.GetOrgPolicyViolationsPreviewRequest, dict]]):
                The request object. GetOrgPolicyViolationsPreviewRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview].
            name (:class:`str`):
                Required. The name of the
                OrgPolicyViolationsPreview to get.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview:
                OrgPolicyViolationsPreview is a resource providing a preview of the
                   violations that will exist if an OrgPolicy change is
                   made.

                   The list of violations are modeled as child resources
                   and retrieved via a [ListOrgPolicyViolations][] API
                   call. There are potentially more
                   [OrgPolicyViolations][] than could fit in an embedded
                   field. Thus, the use of a child resource instead of a
                   field.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest):
            request = gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_org_policy_violations_preview
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def create_org_policy_violations_preview(
        self,
        request: Optional[
            Union[gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        org_policy_violations_preview: Optional[
            gcp_orgpolicy.OrgPolicyViolationsPreview
        ] = None,
        org_policy_violations_preview_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""CreateOrgPolicyViolationsPreview creates an
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        for the proposed changes in the provided
        [OrgPolicyViolationsPreview.OrgPolicyOverlay][]. The changes to
        OrgPolicy are specified by this ``OrgPolicyOverlay``. The
        resources to scan are inferred from these specified changes.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_create_org_policy_violations_preview():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.CreateOrgPolicyViolationsPreviewRequest(
                    parent="parent_value",
                )

                # Make the request
                operation = client.create_org_policy_violations_preview(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.CreateOrgPolicyViolationsPreviewRequest, dict]]):
                The request object. CreateOrgPolicyViolationsPreviewRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview].
            parent (:class:`str`):
                Required. The organization under which this
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
                will be created.

                Example:
                ``organizations/my-example-org/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            org_policy_violations_preview (:class:`google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview`):
                Required. The
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
                to generate.

                This corresponds to the ``org_policy_violations_preview`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            org_policy_violations_preview_id (:class:`str`):
                Optional. An optional user-specified ID for the
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].
                If not provided, a random ID will be generated.

                This corresponds to the ``org_policy_violations_preview_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview` OrgPolicyViolationsPreview is a resource providing a preview of the
                   violations that will exist if an OrgPolicy change is
                   made.

                   The list of violations are modeled as child resources
                   and retrieved via a [ListOrgPolicyViolations][] API
                   call. There are potentially more
                   [OrgPolicyViolations][] than could fit in an embedded
                   field. Thus, the use of a child resource instead of a
                   field.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [
            parent,
            org_policy_violations_preview,
            org_policy_violations_preview_id,
        ]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest
        ):
            request = gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if org_policy_violations_preview is not None:
            request.org_policy_violations_preview = org_policy_violations_preview
        if org_policy_violations_preview_id is not None:
            request.org_policy_violations_preview_id = org_policy_violations_preview_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.create_org_policy_violations_preview
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            gcp_orgpolicy.OrgPolicyViolationsPreview,
            metadata_type=gcp_orgpolicy.CreateOrgPolicyViolationsPreviewOperationMetadata,
        )

        # Done; return the response.
        return response

    async def list_org_policy_violations(
        self,
        request: Optional[
            Union[gcp_orgpolicy.ListOrgPolicyViolationsRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListOrgPolicyViolationsAsyncPager:
        r"""ListOrgPolicyViolations lists the [OrgPolicyViolations][] that
        are present in an
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_list_org_policy_violations():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.ListOrgPolicyViolationsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_org_policy_violations(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.ListOrgPolicyViolationsRequest, dict]]):
                The request object. ListOrgPolicyViolationsRequest is the request message
                for
                [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations].
            parent (:class:`str`):
                Required. The
                OrgPolicyViolationsPreview to get
                OrgPolicyViolations from. Format:

                organizations/{organization}/locations/{location}/orgPolicyViolationsPreviews/{orgPolicyViolationsPreview}

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.pagers.ListOrgPolicyViolationsAsyncPager:
                ListOrgPolicyViolationsResponse is the response message for
                   [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations]

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, gcp_orgpolicy.ListOrgPolicyViolationsRequest):
            request = gcp_orgpolicy.ListOrgPolicyViolationsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_org_policy_violations
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListOrgPolicyViolationsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_operations(
        self,
        request: Optional[Union[operations_pb2.ListOperationsRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operations_pb2.ListOperationsResponse:
        r"""Lists operations that match the specified filter in the request.

        Args:
            request (:class:`~.operations_pb2.ListOperationsRequest`):
                The request object. Request message for
                `ListOperations` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        Returns:
            ~.operations_pb2.ListOperationsResponse:
                Response message for ``ListOperations`` method.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if request is None:
            request_pb = operations_pb2.ListOperationsRequest()
        elif isinstance(request, dict):
            request_pb = operations_pb2.ListOperationsRequest(**request)
        else:
            request_pb = request

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self.transport._wrapped_methods[self._client._transport.list_operations]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request_pb.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request_pb,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_operation(
        self,
        request: Optional[Union[operations_pb2.GetOperationRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if request is None:
            request_pb = operations_pb2.GetOperationRequest()
        elif isinstance(request, dict):
            request_pb = operations_pb2.GetOperationRequest(**request)
        else:
            request_pb = request

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self.transport._wrapped_methods[self._client._transport.get_operation]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request_pb.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request_pb,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "OrgPolicyViolationsPreviewServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("OrgPolicyViolationsPreviewServiceAsyncClient",)
